@@ -9,11 +9,15 @@ import (
 	_ "github.com/wader/fq/format/av1"
 	_ "github.com/wader/fq/format/avro"
 	_ "github.com/wader/fq/format/bencode"
+	_ "github.com/wader/fq/format/bitcoin"
 	_ "github.com/wader/fq/format/bson"
 	_ "github.com/wader/fq/format/bzip2"
 	_ "github.com/wader/fq/format/cbor"
+	_ "github.com/wader/fq/format/crypto"
+	_ "github.com/wader/fq/format/csv"
 	_ "github.com/wader/fq/format/dns"
 	_ "github.com/wader/fq/format/elf"
+	_ "github.com/wader/fq/format/fairplay"
 	_ "github.com/wader/fq/format/flac"
 	_ "github.com/wader/fq/format/gif"
 	_ "github.com/wader/fq/format/gzip"
@@ -23,6 +27,7 @@ import (
 	_ "github.com/wader/fq/format/jpeg"
 	_ "github.com/wader/fq/format/json"
 	_ "github.com/wader/fq/format/macho"
+	_ "github.com/wader/fq/format/math"
 	_ "github.com/wader/fq/format/matroska"
 	_ "github.com/wader/fq/format/mp3"
 	_ "github.com/wader/fq/format/mp4"
@@ -36,10 +41,14 @@ import (
 	_ "github.com/wader/fq/format/raw"
 	_ "github.com/wader/fq/format/rtmp"
 	_ "github.com/wader/fq/format/tar"
+	_ "github.com/wader/fq/format/text"
 	_ "github.com/wader/fq/format/tiff"
+	_ "github.com/wader/fq/format/toml"
 	_ "github.com/wader/fq/format/vorbis"
 	_ "github.com/wader/fq/format/vpx"
 	_ "github.com/wader/fq/format/wav"
 	_ "github.com/wader/fq/format/webp"
+	_ "github.com/wader/fq/format/xml"
+	_ "github.com/wader/fq/format/yaml"
 	_ "github.com/wader/fq/format/zip"
 )
