@@ -15,7 +15,7 @@ func decodeFixedFn(schema schema.SimplifiedSchema, sms ...scalar.Mapper) (Decode
 	}
 	size := int64(schema.Size)
 	// Fixed instances are encoded using the number of bytes declared in the schema.
-	return func(name string, d *decode.D) interface{} {
+	return func(name string, d *decode.D) any {
 		r := d.FieldRawLen(name, size*8, sms...)
 		val := make([]byte, size)
 		if _, err := r.ReadBits(val, size*8); err != nil {
