@@ -2,19 +2,19 @@ package dns
 
 import (
 	"github.com/wader/fq/format"
-	"github.com/wader/fq/format/registry"
 	"github.com/wader/fq/pkg/decode"
+	"github.com/wader/fq/pkg/interp"
 )
 
 func init() {
-	registry.MustRegister(decode.Format{
+	interp.RegisterFormat(decode.Format{
 		Name:        format.DNS_TCP,
 		Description: "DNS packet (TCP)",
 		DecodeFn:    dnsTCPDecode,
 	})
 }
 
-func dnsTCPDecode(d *decode.D, in interface{}) interface{} {
+func dnsTCPDecode(d *decode.D, in any) any {
 	if tsi, ok := in.(format.TCPStreamIn); ok {
 		tsi.MustIsPort(d.Fatalf, format.TCPPortDomain, format.TCPPortDomain)
 	}
