@@ -7,15 +7,15 @@ import (
 	"fmt"
 
 	"github.com/wader/fq/format"
-	"github.com/wader/fq/format/registry"
 	"github.com/wader/fq/pkg/decode"
+	"github.com/wader/fq/pkg/interp"
 	"github.com/wader/fq/pkg/scalar"
 )
 
 var ether8023FrameInetPacketGroup decode.Group
 
 func init() {
-	registry.MustRegister(decode.Format{
+	interp.RegisterFormat(decode.Format{
 		Name:        format.ETHER8023_FRAME,
 		Description: "Ethernet 802.3 frame",
 		Groups:      []string{format.LINK_FRAME},
@@ -34,16 +34,16 @@ var mapUToEtherSym = scalar.Fn(func(s scalar.S) (scalar.S, error) {
 	return s, nil
 })
 
-func decodeEthernetFrame(d *decode.D, in interface{}) interface{} {
+func decodeEthernetFrame(d *decode.D, in any) any {
 	if lfi, ok := in.(format.LinkFrameIn); ok {
 		if lfi.Type != format.LinkTypeETHERNET {
 			d.Fatalf("wrong link type %d", lfi.Type)
 		}
 	}
 
-	d.FieldU("destination", 48, mapUToEtherSym, scalar.Hex)
-	d.FieldU("source", 48, mapUToEtherSym, scalar.Hex)
-	etherType := d.FieldU16("ether_type", format.EtherTypeMap, scalar.Hex)
+	d.FieldU("destination", 48, mapUToEtherSym, scalar.ActualHex)
+	d.FieldU("source", 48, mapUToEtherSym, scalar.ActualHex)
+	etherType := d.FieldU16("ether_type", format.EtherTypeMap, scalar.ActualHex)
 
 	d.FieldFormatOrRawLen(
 		"payload",
