@@ -5,8 +5,8 @@ import (
 	"fmt"
 
 	"github.com/wader/fq/format"
-	"github.com/wader/fq/format/registry"
 	"github.com/wader/fq/pkg/decode"
+	"github.com/wader/fq/pkg/interp"
 	"github.com/wader/fq/pkg/scalar"
 )
 
@@ -14,7 +14,7 @@ var mpegASCFormat decode.Group
 var vorbisPacketFormat decode.Group
 
 func init() {
-	registry.MustRegister(decode.Format{
+	interp.RegisterFormat(decode.Format{
 		Name:        format.MPEG_ES,
 		Description: "MPEG Elementary Stream",
 		DecodeFn:    esDecode,
@@ -161,7 +161,8 @@ func esLengthEncoding(d *decode.D) uint64 {
 	return v
 }
 
-func fieldODDecodeTag(d *decode.D, edc *esDecodeContext, name string, expectedTagID int, fn func(d *decode.D)) { //nolint:unparam
+//nolint:unparam
+func fieldODDecodeTag(d *decode.D, edc *esDecodeContext, name string, expectedTagID int, fn func(d *decode.D)) {
 	d.FieldStruct(name, func(d *decode.D) {
 		odDecodeTag(d, edc, expectedTagID, fn)
 	})
@@ -172,7 +173,7 @@ type esDecodeContext struct {
 	decoderConfigs       []format.MpegDecoderConfig
 }
 
-func odDecodeTag(d *decode.D, edc *esDecodeContext, expectedTagID int, fn func(d *decode.D)) { //nolint:unparam
+func odDecodeTag(d *decode.D, edc *esDecodeContext, _ int, fn func(d *decode.D)) {
 	odDecoders := map[uint64]func(d *decode.D){
 		ES_DescrTag: func(d *decode.D) {
 			d.FieldU16("es_id")
@@ -276,7 +277,7 @@ func odDecodeTag(d *decode.D, edc *esDecodeContext, expectedTagID int, fn func(d
 	}
 }
 
-func esDecode(d *decode.D, in interface{}) interface{} {
+func esDecode(d *decode.D, _ any) any {
 	var edc esDecodeContext
 	odDecodeTag(d, &edc, -1, nil)
 	return format.MpegEsOut{DecoderConfigs: edc.decoderConfigs}
