@@ -2,14 +2,14 @@ package raw
 
 import (
 	"github.com/wader/fq/format"
-	"github.com/wader/fq/format/registry"
 	"github.com/wader/fq/pkg/decode"
+	"github.com/wader/fq/pkg/interp"
 )
 
 func init() {
-	registry.MustRegister(decode.Format{
+	interp.RegisterFormat(decode.Format{
 		Name:        format.RAW,
 		Description: "Raw bits",
-		DecodeFn:    func(d *decode.D, in interface{}) interface{} { return nil },
+		DecodeFn:    func(d *decode.D, _ any) any { return nil },
 	})
 }
