@@ -1,6 +1,6 @@
 package bitioextra
 
-// bitio helpers that don't really belong in bitio
+// bitio helpers that im not sure belong in bitio
 
 import (
 	"errors"
@@ -17,14 +17,6 @@ func CopyBits(dst io.Writer, src bitio.Reader) (int64, error) {
 	return CopyBitsBuffer(dst, src, nil)
 }
 
-func Clone(br bitio.ReaderAtSeeker) (bitio.ReaderAtSeeker, error) {
-	l, err := Len(br)
-	if err != nil {
-		return nil, err
-	}
-	return bitio.NewSectionReader(br, 0, l), nil
-}
-
 func Len(br bitio.ReadAtSeeker) (int64, error) {
 	bPos, err := br.SeekBits(0, io.SeekCurrent)
 	if err != nil {
