@@ -10,10 +10,9 @@ import (
 	"github.com/wader/fq/internal/difftest"
 	"github.com/wader/fq/internal/script"
 	"github.com/wader/fq/pkg/interp"
-	"github.com/wader/fq/pkg/registry"
 )
 
-func TestPath(t *testing.T, registry *registry.Registry) {
+func TestPath(t *testing.T, registry *interp.Registry) {
 	difftest.TestWithOptions(t, difftest.Options{
 		Path:        ".",
 		Pattern:     "*.fqtest",
