@@ -12,10 +12,10 @@ import (
 	"time"
 
 	"github.com/mitchellh/copystructure"
-	"github.com/mitchellh/mapstructure"
 	"github.com/wader/fq/internal/bitioextra"
 	"github.com/wader/fq/internal/gojqextra"
 	"github.com/wader/fq/internal/ioextra"
+	"github.com/wader/fq/internal/mapstruct"
 	"github.com/wader/fq/pkg/bitio"
 	"github.com/wader/fq/pkg/decode"
 	"github.com/wader/fq/pkg/scalar"
@@ -24,13 +24,9 @@ import (
 )
 
 func init() {
-	functionRegisterFns = append(functionRegisterFns, func(i *Interp) []Function {
-		return []Function{
-			{"_registry", 0, 0, i._registry, nil},
-			{"_tovalue", 1, 1, i._toValue, nil},
-			{"_decode", 2, 2, i._decode, nil},
-		}
-	})
+	RegisterFunc0("_registry", (*Interp)._registry)
+	RegisterFunc1("_tovalue", (*Interp)._toValue)
+	RegisterFunc2("_decode", (*Interp)._decode)
 }
 
 type expectedExtkeyError struct {
@@ -50,16 +46,16 @@ type DecodeValue interface {
 	DecodeValue() *decode.Value
 }
 
-func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
+func (i *Interp) _registry(c any) any {
 	uniqueFormats := map[string]decode.Format{}
 
-	groups := map[string]interface{}{}
-	formats := map[string]interface{}{}
+	groups := map[string]any{}
+	formats := map[string]any{}
 
-	for fsName := range i.registry.Groups {
-		var group []interface{}
+	for groupName := range i.Registry.FormatGroups {
+		var group []any
 
-		for _, f := range i.registry.MustGroup(fsName) {
+		for _, f := range i.Registry.MustFormatGroup(groupName) {
 			group = append(group, f.Name)
 			if _, ok := uniqueFormats[f.Name]; ok {
 				continue
@@ -67,11 +63,11 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 			uniqueFormats[f.Name] = f
 		}
 
-		groups[fsName] = group
+		groups[groupName] = group
 	}
 
 	for _, f := range uniqueFormats {
-		vf := map[string]interface{}{
+		vf := map[string]any{
 			"name":        f.Name,
 			"description": f.Description,
 			"probe_order": f.ProbeOrder,
@@ -79,9 +75,9 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 			"root_array":  f.RootArray,
 		}
 
-		var dependenciesVs []interface{}
+		var dependenciesVs []any
 		for _, d := range f.Dependencies {
-			var dNamesVs []interface{}
+			var dNamesVs []any
 			for _, n := range d.Names {
 				dNamesVs = append(dNamesVs, n)
 			}
@@ -90,7 +86,7 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 		if len(dependenciesVs) > 0 {
 			vf["dependencies"] = dependenciesVs
 		}
-		var groupsVs []interface{}
+		var groupsVs []any
 		for _, n := range f.Groups {
 			groupsVs = append(groupsVs, n)
 		}
@@ -98,17 +94,17 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 			vf["groups"] = groupsVs
 		}
 		if f.DecodeInArg != nil {
-			doc := map[string]interface{}{}
+			doc := map[string]any{}
 			st := reflect.TypeOf(f.DecodeInArg)
 			for i := 0; i < st.NumField(); i++ {
 				f := st.Field(i)
 				if v, ok := f.Tag.Lookup("doc"); ok {
-					doc[camelToSnake(f.Name)] = v
+					doc[mapstruct.CamelToSnake(f.Name)] = v
 				}
 			}
 			vf["decode_in_arg_doc"] = doc
 
-			args, err := structToMap(f.DecodeInArg)
+			args, err := mapstruct.ToMap(f.DecodeInArg)
 			if err != nil {
 				return err
 			}
@@ -119,11 +115,11 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 					delete(args, k)
 				}
 			}
-			vf["decode_in_arg"] = args
+			vf["decode_in_arg"] = gojqextra.Normalize(args)
 		}
 
 		if f.Files != nil {
-			files := []interface{}{}
+			files := []any{}
 
 			entries, err := f.Files.ReadDir(".")
 			if err != nil {
@@ -140,7 +136,7 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 					return err
 				}
 
-				files = append(files, map[string]interface{}{
+				files = append(files, map[string]any{
 					"name": e.Name(),
 					"data": string(b),
 				})
@@ -150,7 +146,7 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 		}
 
 		if f.Functions != nil {
-			var ss []interface{}
+			var ss []any
 			for _, f := range f.Functions {
 				ss = append(ss, f)
 			}
@@ -160,28 +156,56 @@ func (i *Interp) _registry(c interface{}, a []interface{}) interface{} {
 		formats[f.Name] = vf
 	}
 
-	return map[string]interface{}{
+	var files []any
+	for _, fs := range i.Registry.FSs {
+		ventries := []any{}
+
+		entries, err := fs.ReadDir(".")
+		if err != nil {
+			return err
+		}
+
+		for _, e := range entries {
+			f, err := fs.Open(e.Name())
+			if err != nil {
+				return err
+			}
+			b, err := ioutil.ReadAll(f)
+			if err != nil {
+				return err
+			}
+
+			ventries = append(ventries, map[string]any{
+				"name": e.Name(),
+				"data": string(b),
+			})
+		}
+
+		files = append(files, ventries)
+	}
+
+	return map[string]any{
 		"groups":  groups,
 		"formats": formats,
+		"files":   files,
 	}
 }
 
-func (i *Interp) _toValue(c interface{}, a []interface{}) interface{} {
+func (i *Interp) _toValue(c any, opts map[string]any) any {
 	v, _ := toValue(
-		func() Options { return i.Options(a[0]) },
+		func() Options { return OptionsFromValue(opts) },
 		c,
 	)
 	return v
 }
 
-func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
-	var opts struct {
-		Force    bool                   `mapstructure:"force"`
-		Progress string                 `mapstructure:"_progress"`
-		Remain   map[string]interface{} `mapstructure:",remain"`
-	}
-	_ = mapstructure.Decode(a[1], &opts)
+type decodeOpts struct {
+	Force    bool
+	Progress string
+	Remain   map[string]any `mapstruct:",remain"`
+}
 
+func (i *Interp) _decode(c any, format string, opts decodeOpts) any {
 	var filename string
 
 	// TODO: progress hack
@@ -192,14 +216,14 @@ func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
 		filename = bbf.filename
 
 		if opts.Progress != "" {
-			evalProgress := func(c interface{}) {
+			evalProgress := func(c any) {
 				// {approx_read_bytes: 123, total_size: 123} | opts.Progress
 				_, _ = i.EvalFuncValues(
-					i.evalInstance.ctx,
+					i.EvalInstance.Ctx,
 					c,
 					opts.Progress,
 					nil,
-					EvalOpts{output: ioextra.DiscardCtxWriter{Ctx: i.evalInstance.ctx}},
+					EvalOpts{output: ioextra.DiscardCtxWriter{Ctx: i.EvalInstance.Ctx}},
 				)
 			}
 			lastProgress := time.Now()
@@ -211,7 +235,7 @@ func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
 				}
 				lastProgress = n
 				evalProgress(
-					map[string]interface{}{
+					map[string]any{
 						"approx_read_bytes": approxReadBytes,
 						"total_size":        totalSize,
 					},
@@ -230,23 +254,23 @@ func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
 		return err
 	}
 
-	formatName, err := toString(a[0])
+	formatName, err := toString(format)
 	if err != nil {
 		return err
 	}
-	decodeFormat, err := i.registry.Group(formatName)
+	decodeFormat, err := i.Registry.FormatGroup(formatName)
 	if err != nil {
 		return err
 	}
 
-	dv, formatOut, err := decode.Decode(i.evalInstance.ctx, bv.br, decodeFormat,
+	dv, formatOut, err := decode.Decode(i.EvalInstance.Ctx, bv.br, decodeFormat,
 		decode.Options{
 			IsRoot:      true,
 			FillGaps:    true,
 			Force:       opts.Force,
 			Range:       bv.r,
 			Description: filename,
-			FormatInArgFn: func(f decode.Format) (interface{}, error) {
+			FormatInArgFn: func(f decode.Format) (any, error) {
 				inArg := f.DecodeInArg
 				if inArg == nil {
 					return nil, nil
@@ -259,9 +283,8 @@ func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
 				}
 
 				if len(opts.Remain) > 0 {
-					if err := mapToStruct(opts.Remain, &inArg); err != nil {
+					if err := mapstruct.ToStruct(opts.Remain, &inArg); err != nil {
 						// TODO: currently ignores failed struct mappings
-						//nolint: nilerr
 						return f.DecodeInArg, nil
 					}
 				}
@@ -273,7 +296,7 @@ func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
 	if dv == nil {
 		var decodeFormatsErr decode.FormatsError
 		if errors.As(err, &decodeFormatsErr) {
-			var vs []interface{}
+			var vs []any
 			for _, fe := range decodeFormatsErr.Errs {
 				vs = append(vs, fe.Value())
 			}
@@ -284,10 +307,10 @@ func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
 		return valueError{err}
 	}
 
-	var formatOutMap interface{}
+	var formatOutMap any
 
 	if formatOut != nil {
-		formatOutMap, err = structToMap(formatOut)
+		formatOutMap, err = mapstruct.ToMap(formatOut)
 		if err != nil {
 			return err
 		}
@@ -296,13 +319,13 @@ func (i *Interp) _decode(c interface{}, a []interface{}) interface{} {
 	return makeDecodeValueOut(dv, formatOutMap)
 }
 
-func valueKey(name string, a, b func(name string) interface{}) interface{} {
+func valueKey(name string, a, b func(name string) any) any {
 	if strings.HasPrefix(name, "_") {
 		return a(name)
 	}
 	return b(name)
 }
-func valueHas(key interface{}, a func(name string) interface{}, b func(key interface{}) interface{}) interface{} {
+func valueHas(key any, a func(name string) any, b func(key any) any) any {
 	stringKey, ok := key.(string)
 	if ok && strings.HasPrefix(stringKey, "_") {
 		if err, ok := a(stringKey).(error); ok {
@@ -314,24 +337,27 @@ func valueHas(key interface{}, a func(name string) interface{}, b func(key inter
 }
 
 // optsFn is a function as toValue is used by tovalue/0 so needs to be fast
-func toValue(optsFn func() Options, v interface{}) (interface{}, bool) {
+func toValue(optsFn func() Options, v any) (any, bool) {
 	switch v := v.(type) {
 	case JQValueEx:
+		if optsFn == nil {
+			return v.JQValueToGoJQ(), true
+		}
 		return v.JQValueToGoJQEx(optsFn), true
 	case gojq.JQValue:
 		return v.JQValueToGoJQ(), true
-	case nil, bool, float64, int, string, *big.Int, map[string]interface{}, []interface{}:
+	case nil, bool, float64, int, string, *big.Int, map[string]any, []any:
 		return v, true
 	default:
 		return nil, false
 	}
 }
 
-func makeDecodeValue(dv *decode.Value) interface{} {
+func makeDecodeValue(dv *decode.Value) any {
 	return makeDecodeValueOut(dv, nil)
 }
 
-func makeDecodeValueOut(dv *decode.Value, out interface{}) interface{} {
+func makeDecodeValueOut(dv *decode.Value, out any) any {
 	switch vv := dv.V.(type) {
 	case *decode.Compound:
 		if vv.IsArray {
@@ -350,7 +376,7 @@ func makeDecodeValueOut(dv *decode.Value, out interface{}) interface{} {
 					IsScalar: true,
 					Fn: func() (gojq.JQValue, error) {
 						buf := &bytes.Buffer{}
-						vvC, err := bitioextra.Clone(vv)
+						vvC, err := bitio.CloneReader(vv)
 						if err != nil {
 							return nil, err
 						}
@@ -393,12 +419,12 @@ func makeDecodeValueOut(dv *decode.Value, out interface{}) interface{} {
 				JQValue:         gojqextra.String(vv),
 				decodeValueBase: decodeValueBase{dv: dv},
 			}
-		case []interface{}:
+		case []any:
 			return decodeValue{
 				JQValue:         gojqextra.Array(vv),
 				decodeValueBase: decodeValueBase{dv: dv},
 			}
-		case map[string]interface{}:
+		case map[string]any:
 			return decodeValue{
 				JQValue:         gojqextra.Object(vv),
 				decodeValueBase: decodeValueBase{dv: dv},
@@ -423,7 +449,7 @@ func makeDecodeValueOut(dv *decode.Value, out interface{}) interface{} {
 
 type decodeValueBase struct {
 	dv  *decode.Value
-	out interface{}
+	out any
 }
 
 func (dvb decodeValueBase) DecodeValue() *decode.Value {
@@ -470,7 +496,7 @@ func (dvb decodeValueBase) ExtKeys() []string {
 	return kv
 }
 
-func (dvb decodeValueBase) JQValueKey(name string) interface{} {
+func (dvb decodeValueBase) JQValueKey(name string) any {
 	dv := dvb.dv
 
 	switch name {
@@ -535,17 +561,11 @@ func (dvb decodeValueBase) JQValueKey(name string) interface{} {
 	case "_path":
 		return valuePath(dv)
 	case "_error":
-		switch vv := dv.V.(type) {
-		case *decode.Compound:
-			var formatErr decode.FormatError
-			if errors.As(vv.Err, &formatErr) {
-				return formatErr.Value()
-
-			}
-			return vv.Err
-		default:
-			return nil
+		var formatErr decode.FormatError
+		if errors.As(dv.Err, &formatErr) {
+			return formatErr.Value()
 		}
+		return nil
 	case "_bits":
 		return Binary{
 			br:   dv.RootReader,
@@ -559,23 +579,10 @@ func (dvb decodeValueBase) JQValueKey(name string) interface{} {
 			unit: 8,
 		}
 	case "_format":
-		switch vv := dv.V.(type) {
-		case *decode.Compound:
-			if vv.Format != nil {
-				return vv.Format.Name
-			}
-			return nil
-		case *scalar.S:
-			// TODO: hack, Scalar interface?
-			switch vv.Actual.(type) {
-			case map[string]interface{}, []interface{}:
-				return "json"
-			default:
-				return nil
-			}
-		default:
-			return nil
+		if dv.Format != nil {
+			return dv.Format.Name
 		}
+		return nil
 	case "_out":
 		return dvb.out
 	case "_unknown":
@@ -602,13 +609,13 @@ type decodeValue struct {
 	bitsFormat bool
 }
 
-func (v decodeValue) JQValueKey(name string) interface{} {
+func (v decodeValue) JQValueKey(name string) any {
 	return valueKey(name, v.decodeValueBase.JQValueKey, v.JQValue.JQValueKey)
 }
-func (v decodeValue) JQValueHas(key interface{}) interface{} {
+func (v decodeValue) JQValueHas(key any) any {
 	return valueHas(key, v.decodeValueBase.JQValueKey, v.JQValue.JQValueHas)
 }
-func (v decodeValue) JQValueToGoJQEx(optsFn func() Options) interface{} {
+func (v decodeValue) JQValueToGoJQEx(optsFn func() Options) any {
 	if !v.bitsFormat {
 		return v.JQValueToGoJQ()
 	}
@@ -622,7 +629,7 @@ func (v decodeValue) JQValueToGoJQEx(optsFn func() Options) interface{} {
 		return err
 	}
 
-	brC, err := bitioextra.Clone(br)
+	brC, err := bitio.CloneReaderAtSeeker(br)
 	if err != nil {
 		return err
 	}
@@ -644,7 +651,7 @@ type ArrayDecodeValue struct {
 	*decode.Compound
 }
 
-func NewArrayDecodeValue(dv *decode.Value, out interface{}, c *decode.Compound) ArrayDecodeValue {
+func NewArrayDecodeValue(dv *decode.Value, out any, c *decode.Compound) ArrayDecodeValue {
 	return ArrayDecodeValue{
 		decodeValueBase: decodeValueBase{dv: dv, out: out},
 		Base:            gojqextra.Base{Typ: "array"},
@@ -652,44 +659,44 @@ func NewArrayDecodeValue(dv *decode.Value, out interface{}, c *decode.Compound)
 	}
 }
 
-func (v ArrayDecodeValue) JQValueKey(name string) interface{} {
+func (v ArrayDecodeValue) JQValueKey(name string) any {
 	return valueKey(name, v.decodeValueBase.JQValueKey, v.Base.JQValueKey)
 }
-func (v ArrayDecodeValue) JQValueSliceLen() interface{} { return len(v.Compound.Children) }
-func (v ArrayDecodeValue) JQValueLength() interface{}   { return len(v.Compound.Children) }
-func (v ArrayDecodeValue) JQValueIndex(index int) interface{} {
+func (v ArrayDecodeValue) JQValueSliceLen() any { return len(v.Compound.Children) }
+func (v ArrayDecodeValue) JQValueLength() any   { return len(v.Compound.Children) }
+func (v ArrayDecodeValue) JQValueIndex(index int) any {
 	// -1 outside after string, -2 outside before string
 	if index < 0 {
 		return nil
 	}
 	return makeDecodeValue((v.Compound.Children)[index])
 }
-func (v ArrayDecodeValue) JQValueSlice(start int, end int) interface{} {
-	vs := make([]interface{}, end-start)
+func (v ArrayDecodeValue) JQValueSlice(start int, end int) any {
+	vs := make([]any, end-start)
 	for i, e := range (v.Compound.Children)[start:end] {
 		vs[i] = makeDecodeValue(e)
 	}
 	return vs
 }
-func (v ArrayDecodeValue) JQValueEach() interface{} {
+func (v ArrayDecodeValue) JQValueEach() any {
 	props := make([]gojq.PathValue, len(v.Compound.Children))
 	for i, f := range v.Compound.Children {
 		props[i] = gojq.PathValue{Path: i, Value: makeDecodeValue(f)}
 	}
 	return props
 }
-func (v ArrayDecodeValue) JQValueKeys() interface{} {
-	vs := make([]interface{}, len(v.Compound.Children))
+func (v ArrayDecodeValue) JQValueKeys() any {
+	vs := make([]any, len(v.Compound.Children))
 	for i := range v.Compound.Children {
 		vs[i] = i
 	}
 	return vs
 }
-func (v ArrayDecodeValue) JQValueHas(key interface{}) interface{} {
+func (v ArrayDecodeValue) JQValueHas(key any) any {
 	return valueHas(
 		key,
 		v.decodeValueBase.JQValueKey,
-		func(key interface{}) interface{} {
+		func(key any) any {
 			intKey, ok := key.(int)
 			if !ok {
 				return gojqextra.HasKeyTypeError{L: "array", R: fmt.Sprintf("%v", key)}
@@ -697,8 +704,8 @@ func (v ArrayDecodeValue) JQValueHas(key interface{}) interface{} {
 			return intKey >= 0 && intKey < len(v.Compound.Children)
 		})
 }
-func (v ArrayDecodeValue) JQValueToGoJQ() interface{} {
-	vs := make([]interface{}, len(v.Compound.Children))
+func (v ArrayDecodeValue) JQValueToGoJQ() any {
+	vs := make([]any, len(v.Compound.Children))
 	for i, f := range v.Compound.Children {
 		vs[i] = makeDecodeValue(f)
 	}
@@ -715,7 +722,7 @@ type StructDecodeValue struct {
 	*decode.Compound
 }
 
-func NewStructDecodeValue(dv *decode.Value, out interface{}, c *decode.Compound) StructDecodeValue {
+func NewStructDecodeValue(dv *decode.Value, out any, c *decode.Compound) StructDecodeValue {
 	return StructDecodeValue{
 		decodeValueBase: decodeValueBase{dv: dv, out: out},
 		Base:            gojqextra.Base{Typ: "object"},
@@ -723,9 +730,9 @@ func NewStructDecodeValue(dv *decode.Value, out interface{}, c *decode.Compound)
 	}
 }
 
-func (v StructDecodeValue) JQValueLength() interface{}   { return len(v.Compound.Children) }
-func (v StructDecodeValue) JQValueSliceLen() interface{} { return len(v.Compound.Children) }
-func (v StructDecodeValue) JQValueKey(name string) interface{} {
+func (v StructDecodeValue) JQValueLength() any   { return len(v.Compound.Children) }
+func (v StructDecodeValue) JQValueSliceLen() any { return len(v.Compound.Children) }
+func (v StructDecodeValue) JQValueKey(name string) any {
 	if strings.HasPrefix(name, "_") {
 		return v.decodeValueBase.JQValueKey(name)
 	}
@@ -737,25 +744,25 @@ func (v StructDecodeValue) JQValueKey(name string) interface{} {
 	}
 	return nil
 }
-func (v StructDecodeValue) JQValueEach() interface{} {
+func (v StructDecodeValue) JQValueEach() any {
 	props := make([]gojq.PathValue, len(v.Compound.Children))
 	for i, f := range v.Compound.Children {
 		props[i] = gojq.PathValue{Path: f.Name, Value: makeDecodeValue(f)}
 	}
 	return props
 }
-func (v StructDecodeValue) JQValueKeys() interface{} {
-	vs := make([]interface{}, len(v.Compound.Children))
+func (v StructDecodeValue) JQValueKeys() any {
+	vs := make([]any, len(v.Compound.Children))
 	for i, f := range v.Compound.Children {
 		vs[i] = f.Name
 	}
 	return vs
 }
-func (v StructDecodeValue) JQValueHas(key interface{}) interface{} {
+func (v StructDecodeValue) JQValueHas(key any) any {
 	return valueHas(
 		key,
 		v.decodeValueBase.JQValueKey,
-		func(key interface{}) interface{} {
+		func(key any) any {
 			stringKey, ok := key.(string)
 			if !ok {
 				return gojqextra.HasKeyTypeError{L: "object", R: fmt.Sprintf("%v", key)}
@@ -769,8 +776,8 @@ func (v StructDecodeValue) JQValueHas(key interface{}) interface{} {
 		},
 	)
 }
-func (v StructDecodeValue) JQValueToGoJQ() interface{} {
-	vm := make(map[string]interface{}, len(v.Compound.Children))
+func (v StructDecodeValue) JQValueToGoJQ() any {
+	vm := make(map[string]any, len(v.Compound.Children))
 	for _, f := range v.Compound.Children {
 		vm[f.Name] = makeDecodeValue(f)
 	}
